#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import scapy

""" Este script carga un archivo pcap guardado por ejemplo con Wireshark y 
    reconstruir sus paquetes con Scapy, para cualquier objetivo 
    (por ejemplo, sirve para hacer replay de algunas tramas, paquetes o 
    segmentos, a cualquier nivel).
    
    En este caso en particular, el main itera por cada paquete del pcap y 
    llama a una funcion "print_packet" para que la imprima por stdout.

    La ventaja contra el export de Wireshark es que puedo imprimir 
    cualquier campo a cualquier nivel de cada paquete, obteniendo un control
    total del proceso. Surgió como necesidad de formatear un Idle Scan, donde
    era necesario mostrar el IP-ID de cada paquete del Zombie.
    
    Tecnica para templates sencillos:
    http://www.noah.org/wiki/Python_templates

    Relacionado:
    - Para ver paquetes obtenidos con scapy con wireshark:
      http://trac.secdev.org/scapy/wiki/ViewingPacketsWithWireshark

    - Si se obtiene esto por consola (Ubuntu 7.10 _x86-64):

    In [1]: from scapy import *
    ERROR: Error during evaluation of config file [None]
    Traceback (most recent call last):
      File "/var/lib/python-support/python2.5/scapy.py", line 12185, in read_config_file
          execfile(configfile)
          TypeError: execfile() argument 1 must be string, not None

    Crear el archivo ~/.scapy_startup.py, con el comando "touch ~/.scapy_startup.py"

    Autor: Marcelo Fernández
"""

# Escapo todos los campos que quiero que paresee scapy con %%
TEMPLATE = """Paquete nro. %s - Tiempo %f
Mac Origen: %%Ether.src%% -> Mac Destino: %%Ether.dst%%
IP Origen: %%IP.src%% -> %%IP.dst%%
IP Header: Version:%%IP.version%% Len:%%IP.len%% ID:%%IP.id%% Frag:%%IP.frag%% TTL:%%IP.ttl%% (...)
TCP Header: SPort:%%r,TCP.sport%% DPort:%%r,TCP.dport%% SeqN:%%TCP.seq%% AckN:%%TCP.ack%% Flags:0x%%02xr,TCP.flags%%(%%TCP.flags%%) Wnd:%%TCP.window%% (...)
Descripcion: %s
""" 

TEMPLATE_ICMP = """Paquete nro. %s - Tiempo %f
IP Origen: %%IP.src%% -> %%IP.dst%%
IP Header: Version:%%IP.version%% Proto:0x%%02xr,IP.proto%% (%%IP.proto%%) (...)
ICMP Header: Tipo:0x%%02xr,ICMP.type%% Codigo:%%ICMP.code%%
Descripcion: %s
""" 

def print_packets(paquetes):
    """ Esta funcion recibe una lista de instancias de scapy.Ether y los imprime
        de acuerdo a la variable template. """
    
    # El formato Pcap guarda el tiempo desde el Epoch, a mí no me gusta, prefiero
    # en relación a la primera trama capturada
    zero_time = paquetes[0].time
    index = 0
    
    for p in paquetes:
        index += 1
        relative_time = p.time - zero_time
        # Armo un template dinámico para pasarle al sprintf()
        
        """ Está disponible también la keyname prn para imprimir algo en 
            particular y la lfilter para filtrar por algún parámetro
            paquetes.summary(prn = lambda(p): p.fields)
            paquetes.summary(lfilter = lambda(p): p[TCP].src == "192.168.0.2")"""
        
        template = TEMPLATE_ICMP % (index, relative_time, p.summary())
        print p.sprintf(template)


if __name__ == '__main__':
    if len(sys.argv) > 1:
        # Tengo un parámetro, debe ser el archivo
        archivo_pcap = sys.argv[1]
    else:
        print "Modo de uso: python pcap_parser.py archivo_pcap.cap"
        sys.exit(1)

    try:
        paquetes = scapy.rdpcap(archivo_pcap)
    except IOError:
        print "Archivo %s no encontrado" % archivo_pcap
        sys.exit(2)

    print_packets(paquetes)
